/* Copyright (c) 2016 Open-RnD Sp. z o.o.
 * Copyright (c) 2020 Nordic Semiconductor ASA
 * SPDX-License-Identifier: Apache-2.0 */

#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/util.h>
#include <zephyr/sys/printk.h>
#include <inttypes.h>

//SELECT BUTTON PIN HERE (either line13 or line14)
//static const struct gpio_dt_spec button  = GPIO_DT_SPEC_GET_OR(DT_ALIAS(sw0), gpios, {0});	        //SW0 as button
static const struct gpio_dt_spec button = GPIO_DT_SPEC_GET_OR(DT_NODELABEL(nslpin), gpios, {0});        //nslpin as button

static struct gpio_callback button_cb_data;																//cb

void button_pressed(const struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
	printk("Button pressed at %" PRIu32 "\n", k_cycle_get_32());
}

void main(void)
{
	int ret; 

	if (!device_is_ready(button.port)) {printk("Error: button device %s is not ready\n",button.port->name);return;}

	//Configure as INPUT	
	ret = gpio_pin_configure_dt(&button, GPIO_INPUT);
	if (ret != 0) {printk("Error %d: failed to configure %s pin %d\n", ret, button.port->name, button.pin);return;}

	//Configure Interrupt
	ret = gpio_pin_interrupt_configure_dt(&button,GPIO_INT_EDGE_TO_ACTIVE);
	if (ret != 0) {printk("Error %d: failed to configure interrupt on %s pin %d\n",ret, button.port->name, button.pin);return;}
	gpio_init_callback(&button_cb_data, button_pressed, BIT(button.pin));
	gpio_add_callback(button.port, &button_cb_data);
	printk("Set up button at %s pin %d\n", button.port->name, button.pin);

	printk("\n\nButton Configured as Input and INT Enabled... check using nrfjprog ...");
	ret = 10;
	while(ret-- != 0){
		printk("%2d\b\b", ret);
		k_msleep(1000);
	}

	//Disable Interrupt
	ret = gpio_pin_interrupt_configure_dt(&button, GPIO_INT_DISABLE);
	if (ret != 0) {printk("Error %d: failed to DISABLE interrupt on %s pin %d\n",ret, button.port->name, button.pin);return;}	
	printk("\nINT Disabled\n");

	//Configure as OUTPUT
	ret = gpio_pin_configure_dt(&button, GPIO_OUTPUT);
	if (ret != 0) {printk("Error %d: failed to configure %s pin %d\n", ret, button.port->name, button.pin);return;}	
	
	printk("\n\nButton Configured as OUTPUT ... check using nrfjprog ...");
	ret = 10;
	while(ret-- != 0){
		printk("%2d\b\b", ret);
		k_msleep(1000);
	}

	//Setting output to 1
	ret = gpio_pin_set_dt(&button, 1);
	if (ret != 0) {printk("Error %d: Failed to set output on %s pin %d\n", ret, button.port->name, button.pin);return;}	
	printk("\n\nOutput value set: Check using multimeter ...");
	ret = 10;
	while(ret-- != 0){
		printk("%2d\b\b", ret);
		k_msleep(1000);
	}

	//Setting output to 0
	ret = gpio_pin_set_dt(&button, 0);
	if (ret != 0) {printk("Error %d: Failed to set output on %s pin %d\n", ret, button.port->name, button.pin);return;}	
	printk("\n\nOutput value reset: Check using multimeter ...");
	ret = 10;
	while(ret-- != 0){
		printk("%2d\b\b", ret);
		k_msleep(1000);
	}
}